#include "vymview.h"

#include <iostream>
using namespace std;

#include "mainwindow.h"
#include "mapeditor.h"
#include "treeeditor.h"

extern Main *mainWindow;


VymView::VymView(VymModel *m)
{
	model=m;

	// Create TreeView
	treeEditor=new TreeEditor (model);
	//treeEditor->setModel ((QAbstractItemModel*)model);
	//treeEditor->setMinimumWidth (50);

	treeEditor->setColumnWidth (0,150);
	treeEditor->setAnimated (true);

	proxySelModel=treeEditor->selectionModel();
	selModel=new QItemSelectionModel (model);

	//model->setSelectionModel (proxySelModel);
	model->setSelectionModel (selModel);

	// Create good old MapEditor
	mapEditor=model->getMapEditor();
	if (!mapEditor) mapEditor=new MapEditor (model);

	// Connect selections

		// Proxymodel changed
		connect (
			proxySelModel, SIGNAL (selectionChanged(const QItemSelection &, const QItemSelection &)), 
			this,SLOT (changeProxySelection(const QItemSelection &,const QItemSelection &)));

		// Model changed	
		connect (
			selModel, SIGNAL (selectionChanged(const QItemSelection &, const QItemSelection &)), 
			this,SLOT (changeSelection(const QItemSelection &,const QItemSelection &)));

		connect (
			model, SIGNAL (selectionChanged(const QItemSelection &, const QItemSelection &)),
			mapEditor,SLOT (updateSelection(const QItemSelection &,const QItemSelection &)));
			//FIXME-3 above and below necessary???
		connect (
			selModel, SIGNAL (selectionChanged(const QItemSelection &, const QItemSelection &)), 
			mapEditor,SLOT (updateSelection(const QItemSelection &,const QItemSelection &)));

	// Connect data changed signals	
	connect (
		model, SIGNAL (dataChanged(const QModelIndex &, const QModelIndex &)), 
		mapEditor,SLOT (updateData(const QModelIndex &) ) );

	connect (
		model, SIGNAL (sortFilterChanged (const QString &)),
		treeEditor, SLOT (setSortFilter (const QString &) ) );

	connect (
		model, SIGNAL (noteHasChanged(QModelIndex) ),
		mainWindow, SLOT (updateNoteEditor (QModelIndex) ) );
		
	connect (
		model, SIGNAL (expandAll() ),
		this, SLOT (expandAll () ) );
		
	connect (
		model, SIGNAL (showSelection() ),
		this, SLOT (showSelection() ) );
		

	mapEditor->setAntiAlias (mainWindow->isAliased());
	mapEditor->setSmoothPixmap(mainWindow->hasSmoothPixmapTransform());

	addWidget (treeEditor);
	addWidget (mapEditor);

	// Set geometry
	QList <int> widths;
	widths<<200;
	widths<<600;
	setSizes(widths);
}

VymView::~VymView()
{
	//cout << "Destructor VymView\n";
}

VymModel* VymView::getModel()
{
	return model;
}

MapEditor* VymView::getMapEditor()
{
	return mapEditor;
}

void VymView::initFocus()
{
	mapEditor->setFocus();
}

void VymView::changeSelection (const QItemSelection &newsel, const QItemSelection &oldsel)
{
	// Notify mainwindow to update satellites like NoteEditor, if needed (model==currenModel...)
	mainWindow->changeSelection (model,newsel,oldsel);	// FIXME-5 maybe connect VymModel <-> MainWindow directly?
	// would require to also get current model in mainWindow
	proxySelModel->select (
		treeEditor->getProxyModel()->mapSelectionFromSource (newsel),
		QItemSelectionModel::ClearAndSelect );
	showSelection();
}

void VymView::changeProxySelection (const QItemSelection &newsel, const QItemSelection &oldsel)
{
	// Notify mainwindow to update satellites, but map selection to 
	// original model first

	// Re-emit but map selection first
	model->emitSelectionChanged (treeEditor->getProxyModel()->mapSelectionToSource (newsel));
}

void VymView::expandAll()
{
	treeEditor->expandAll();
}

void VymView::showSelection()
{
	QModelIndex ix=model->getSelectedIndex();
	treeEditor->scrollTo( ix, QAbstractItemView::EnsureVisible);
	mapEditor->scrollTo ( ix);
}

